@@ -1,119 +1,79 @@
 from heapq import (heappush,
                    heapreplace)
-from typing import (Iterator,
+from operator import attrgetter
+from typing import (Callable,
+                    Iterator,
                     List,
+                    Optional,
                     Sequence,
                     Tuple,
                     Type,
                     Union)
 
+from ground.coordinates import to_square_rooter as _to_square_rooter
+from ground.hints import (Coordinate,
+                          Point)
 from reprit.base import generate_repr
 
 from .core import interval as _interval
-from .core.utils import (linear_distance as _linear_distance,
-                         planar_distance as _planar_distance)
-from .hints import (Coordinate,
-                    Interval,
-                    Point)
+from .core.utils import points_distance as _points_distance
+from .hints import Interval
+
+try:
+    from typing import Protocol
+except ImportError:
+    from typing_extensions import Protocol
 
 Item = Tuple[int, Point]
 NIL = None
 
+_PROJECTORS = attrgetter('x'), attrgetter('y')
 
-class Node:
+
+class Node(Protocol):
     """
-    Represents node of *kd*-tree.
+    Interface of *kd*-tree node.
 
-    Can be subclassed for custom metrics definition.
+    Can be implemented for custom metrics definition.
     """
 
-    __slots__ = 'index', 'point', 'axis', 'left', 'right'
-
-    def __init__(self,
-                 index: int,
-                 point: Point,
-                 axis: int,
-                 left: Union['Node', NIL],
-                 right: Union['Node', NIL]) -> None:
-        """
-        Initializes node.
-
-        Time complexity:
-            ``O(1)``
-        Memory complexity:
-            ``O(1)``
-
-        >>> node = Node(5, (-10, 10), 0, NIL, NIL)
-        """
-        self.index = index
-        self.point = point
-        self.axis = axis
-        self.left = left
-        self.right = right
+    def __new__(cls,
+                index: int,
+                point: Point,
+                is_y_axis: bool,
+                left: Union['Node', NIL],
+                right: Union['Node', NIL]) -> 'Node':
+        """Creates node."""
 
     @property
-    def coordinate(self) -> Coordinate:
-        """
-        Returns coordinate on which branching was performed.
-
-        Time complexity:
-            ``O(1)``
-        Memory complexity:
-            ``O(1)``
-
-        >>> node = Node(5, (-10, 10), 0, NIL, NIL)
-        >>> node.coordinate == -10
-        True
-        """
-        return self.point[self.axis]
+    def is_y_axis(self) -> bool:
+        """Checks if the node corresponds to the y-axis."""
 
     @property
     def item(self) -> Item:
-        """
-        Returns underlying index with point.
+        """Returns item of the node."""
 
-        Time complexity:
-            ``O(1)``
-        Memory complexity:
-            ``O(1)``
+    @property
+    def left(self) -> Union['Node', NIL]:
+        """Returns left child of the node."""
 
-        >>> node = Node(5, (-10, 10), 0, NIL, NIL)
-        >>> node.item == (5, (-10, 10))
-        True
-        """
-        return self.index, self.point
+    @property
+    def point(self) -> Point:
+        """Returns point of the node."""
+
+    @property
+    def projector(self) -> Callable[[Point], Coordinate]:
+        """Returns projector of the node."""
+
+    @property
+    def right(self) -> Union['Node', NIL]:
+        """Returns right child of the node."""
 
     def distance_to_point(self, point: Point) -> Coordinate:
-        """
-        Calculates distance to given point.
-
-        Time complexity:
-            ``O(1)``
-        Memory complexity:
-            ``O(1)``
-
-        >>> node = Node(5, (-10, 10), 0, NIL, NIL)
-        >>> node.distance_to_point((-7, 6)) == 5
-        True
-        """
-        return _planar_distance(self.point, point)
+        """Calculates distance to given point."""
 
     def distance_to_coordinate(self, coordinate: Coordinate) -> Coordinate:
-        """
-        Calculates distance to given coordinate.
-
-        Time complexity:
-            ``O(1)``
-        Memory complexity:
-            ``O(1)``
-
-        >>> node = Node(5, (-10, 10), 0, NIL, NIL)
-        >>> node.distance_to_coordinate(-1) == 9
-        True
-        """
-        return _linear_distance(self.coordinate, coordinate)
-
-    __repr__ = generate_repr(__init__)
+        """Calculates distance to given coordinate."""
 
 
 class Tree:
@@ -128,7 +88,7 @@ class Tree:
 
     def __init__(self, points: Sequence[Point],
                  *,
-                 node_cls: Type[Node] = Node) -> None:
+                 node_cls: Optional[Type[Node]] = None) -> None:
         """
         Initializes tree from points.
 
@@ -139,12 +99,16 @@ class Tree:
 
         where ``dimension = len(points[0])``, ``size = len(points)``.
 
-        >>> points = list(zip(range(-5, 6), range(10)))
+        >>> from ground.geometries import to_point_cls
+        >>> Point = to_point_cls()
+        >>> points = list(map(Point, range(-5, 6), range(10)))
         >>> tree = Tree(points)
         """
         self._points = points
-        self._root = _create_node(node_cls, range(len(points)), points,
-                                  len(points[0]), 0)
+        self._root = _create_node(_to_default_node_cls()
+                                  if node_cls is None
+                                  else node_cls,
+                                  range(len(points)), points, False)
 
     __repr__ = generate_repr(__init__)
 
@@ -157,11 +121,6 @@ class Tree:
             ``O(1)``
         Memory complexity:
             ``O(1)``
-
-        >>> points = list(zip(range(-5, 6), range(10)))
-        >>> tree = Tree(points)
-        >>> tree.node_cls is Node
-        True
         """
         return type(self._root)
 
@@ -175,7 +134,9 @@ class Tree:
         Memory complexity:
             ``O(1)``
 
-        >>> points = list(zip(range(-5, 6), range(10)))
+        >>> from ground.geometries import to_point_cls
+        >>> Point = to_point_cls()
+        >>> points = list(map(Point, range(-5, 6), range(10)))
         >>> tree = Tree(points)
         >>> tree.points == points
         True
@@ -201,11 +162,14 @@ class Tree:
         :param point: input point.
         :returns: indices of points in the tree the nearest to the input point.
 
-        >>> points = list(zip(range(-5, 6), range(10)))
+        >>> from ground.geometries import to_point_cls
+        >>> Point = to_point_cls()
+        >>> points = list(map(Point, range(-5, 6), range(10)))
         >>> tree = Tree(points)
-        >>> tree.n_nearest_indices(2, (0, 0)) == [2, 3]
+        >>> tree.n_nearest_indices(2, Point(0, 0)) == [2, 3]
         True
-        >>> tree.n_nearest_indices(len(points), (0, 0)) == range(len(points))
+        >>> (tree.n_nearest_indices(len(points), Point(0, 0))
+        ...  == range(len(points)))
         True
         """
         return ([index for index, _ in self._n_nearest_items(n, point)]
@@ -230,11 +194,14 @@ class Tree:
         :param point: input point.
         :returns: points in the tree the nearest to the input point.
 
-        >>> points = list(zip(range(-5, 6), range(10)))
+        >>> from ground.geometries import to_point_cls
+        >>> Point = to_point_cls()
+        >>> points = list(map(Point, range(-5, 6), range(10)))
         >>> tree = Tree(points)
-        >>> tree.n_nearest_points(2, (0, 0)) == [(-3, 2), (-2, 3)]
+        >>> (tree.n_nearest_points(2, Point(0, 0))
+        ...  == [Point(-3, 2), Point(-2, 3)])
         True
-        >>> tree.n_nearest_points(len(points), (0, 0)) == points
+        >>> tree.n_nearest_points(len(points), Point(0, 0)) == points
         True
         """
         return ([point for _, point in self._n_nearest_items(n, point)]
@@ -261,11 +228,14 @@ class Tree:
         :returns:
             indices with points in the tree the nearest to the input point.
 
-        >>> points = list(zip(range(-5, 6), range(10)))
+        >>> from ground.geometries import to_point_cls
+        >>> Point = to_point_cls()
+        >>> points = list(map(Point, range(-5, 6), range(10)))
         >>> tree = Tree(points)
-        >>> tree.n_nearest_items(2, (0, 0)) == [(2, (-3, 2)), (3, (-2, 3))]
+        >>> (tree.n_nearest_items(2, Point(0, 0))
+        ...  == [(2, Point(-3, 2)), (3, Point(-2, 3))])
         True
-        >>> (tree.n_nearest_items(len(points), (0, 0))
+        >>> (tree.n_nearest_items(len(points), Point(0, 0))
         ...  == list(enumerate(points)))
         True
         """
@@ -274,25 +244,26 @@ class Tree:
                 else list(enumerate(self._points)))
 
     def _n_nearest_items(self, n: int, point: Point) -> List[Item]:
-        candidates = []
+        candidates = []  # type: List[Tuple[Coordinate, Item]]
         queue = [self._root]
         push, pop = queue.append, queue.pop
         while queue:
-            node = pop()
+            node = pop()  # type: Node
             distance_to_point = node.distance_to_point(point)
             candidate = -distance_to_point, node.item
             if len(candidates) < n:
                 heappush(candidates, candidate)
             elif distance_to_point < -candidates[0][0]:
                 heapreplace(candidates, candidate)
-            point_is_on_the_left = point[node.axis] < node.coordinate
+            coordinate = node.projector(point)
+            point_is_on_the_left = coordinate < node.projector(node.point)
             if point_is_on_the_left:
                 if node.left is not NIL:
                     push(node.left)
             elif node.right is not NIL:
                 push(node.right)
             if (len(candidates) < n
-                    or (node.distance_to_coordinate(point[node.axis])
+                    or (node.distance_to_coordinate(coordinate)
                         < -candidates[0][0])):
                 if point_is_on_the_left:
                     if node.right is not NIL:
@@ -319,11 +290,13 @@ class Tree:
         :param point: input point.
         :returns: index of a point in the tree the nearest to the input point.
 
-        >>> points = list(zip(range(-5, 6), range(10)))
+        >>> from ground.geometries import to_point_cls
+        >>> Point = to_point_cls()
+        >>> points = list(map(Point, range(-5, 6), range(10)))
         >>> tree = Tree(points)
-        >>> tree.nearest_index((0, 0)) == 2
+        >>> tree.nearest_index(Point(0, 0)) == 2
         True
-        >>> tree.nearest_index((-3, 2)) == 2
+        >>> tree.nearest_index(Point(-3, 2)) == 2
         True
         """
         result, _ = self.nearest_item(point)
@@ -346,11 +319,13 @@ class Tree:
         :param point: input point.
         :returns: point in the tree the nearest to the input point.
 
-        >>> points = list(zip(range(-5, 6), range(10)))
+        >>> from ground.geometries import to_point_cls
+        >>> Point = to_point_cls()
+        >>> points = list(map(Point, range(-5, 6), range(10)))
         >>> tree = Tree(points)
-        >>> tree.nearest_point((0, 0)) == (-3, 2)
+        >>> tree.nearest_point(Point(0, 0)) == Point(-3, 2)
         True
-        >>> tree.nearest_point((-3, 2)) == (-3, 2)
+        >>> tree.nearest_point(Point(-3, 2)) == Point(-3, 2)
         True
         """
         _, result = self.nearest_item(point)
@@ -374,11 +349,13 @@ class Tree:
         :param point: input point.
         :returns: index with point in the tree the nearest to the input point.
 
-        >>> points = list(zip(range(-5, 6), range(10)))
+        >>> from ground.geometries import to_point_cls
+        >>> Point = to_point_cls()
+        >>> points = list(map(Point, range(-5, 6), range(10)))
         >>> tree = Tree(points)
-        >>> tree.nearest_item((0, 0)) == (2, (-3, 2))
+        >>> tree.nearest_item(Point(0, 0)) == (2, Point(-3, 2))
         True
-        >>> tree.nearest_item((-3, 2)) == (2, (-3, 2))
+        >>> tree.nearest_item(Point(-3, 2)) == (2, Point(-3, 2))
         True
         """
         node = self._root
@@ -386,17 +363,19 @@ class Tree:
         queue = [node]
         push, pop = queue.append, queue.pop
         while queue:
-            node = pop()
+            node = pop()  # type: Node
             distance_to_point = node.distance_to_point(point)
             if distance_to_point < min_distance:
                 result, min_distance = node.item, distance_to_point
-            point_is_on_the_left = point[node.axis] < node.coordinate
+            coordinate, node_coordinate = (node.projector(point),
+                                           node.projector(node.point))
+            point_is_on_the_left = coordinate < node_coordinate
             if point_is_on_the_left:
                 if node.left is not NIL:
                     push(node.left)
             elif node.right is not NIL:
                 push(node.right)
-            if node.distance_to_coordinate(point[node.axis]) < min_distance:
+            if node.distance_to_coordinate(coordinate) < min_distance:
                 if point_is_on_the_left:
                     if node.right is not NIL:
                         push(node.right)
@@ -425,13 +404,15 @@ class Tree:
         :param radius: radius of the ball.
         :returns: indices of points in the tree that lie inside the ball.
 
-        >>> points = list(zip(range(-5, 6), range(10)))
+        >>> from ground.geometries import to_point_cls
+        >>> Point = to_point_cls()
+        >>> points = list(map(Point, range(-5, 6), range(10)))
         >>> tree = Tree(points)
-        >>> tree.find_ball_indices((0, 0), 0) == []
+        >>> tree.find_ball_indices(Point(0, 0), 0) == []
         True
-        >>> tree.find_ball_indices((0, 0), 2) == []
+        >>> tree.find_ball_indices(Point(0, 0), 2) == []
         True
-        >>> tree.find_ball_indices((0, 0), 4) == [2, 3]
+        >>> tree.find_ball_indices(Point(0, 0), 4) == [2, 3]
         True
         """
         return [index for index, _ in self._find_ball_items(center, radius)]
@@ -457,13 +438,16 @@ class Tree:
         :param radius: radius of the ball.
         :returns: points in the tree that lie inside the ball.
 
-        >>> points = list(zip(range(-5, 6), range(10)))
+        >>> from ground.geometries import to_point_cls
+        >>> Point = to_point_cls()
+        >>> points = list(map(Point, range(-5, 6), range(10)))
         >>> tree = Tree(points)
-        >>> tree.find_ball_points((0, 0), 0) == []
+        >>> tree.find_ball_points(Point(0, 0), 0) == []
         True
-        >>> tree.find_ball_points((0, 0), 2) == []
+        >>> tree.find_ball_points(Point(0, 0), 2) == []
         True
-        >>> tree.find_ball_points((0, 0), 4) == [(-3, 2), (-2, 3)]
+        >>> (tree.find_ball_points(Point(0, 0), 4)
+        ...  == [Point(-3, 2), Point(-2, 3)])
         True
         """
         return [point for _, point in self._find_ball_items(center, radius)]
@@ -488,13 +472,16 @@ class Tree:
         :param radius: radius of the ball.
         :returns: indices with points in the tree that lie inside the ball.
 
-        >>> points = list(zip(range(-5, 6), range(10)))
+        >>> from ground.geometries import to_point_cls
+        >>> Point = to_point_cls()
+        >>> points = list(map(Point, range(-5, 6), range(10)))
         >>> tree = Tree(points)
-        >>> tree.find_ball_items((0, 0), 0) == []
+        >>> tree.find_ball_items(Point(0, 0), 0) == []
         True
-        >>> tree.find_ball_items((0, 0), 2) == []
+        >>> tree.find_ball_items(Point(0, 0), 2) == []
         True
-        >>> tree.find_ball_items((0, 0), 4) == [(2, (-3, 2)), (3, (-2, 3))]
+        >>> (tree.find_ball_items(Point(0, 0), 4)
+        ...  == [(2, Point(-3, 2)), (3, Point(-2, 3))])
         True
         """
         return list(self._find_ball_items(center, radius))
@@ -504,10 +491,11 @@ class Tree:
         queue = [self._root]
         push, pop = queue.append, queue.pop
         while queue:
-            node = pop()
+            node = pop()  # type: Node
             if node.distance_to_point(center) <= radius:
                 yield node.item
-            hyperplane_delta = center[node.axis] - node.coordinate
+            hyperplane_delta = (node.projector(center)
+                                - node.projector(node.point))
             if node.left is not NIL and hyperplane_delta <= radius:
                 push(node.left)
             if node.right is not NIL and -radius <= hyperplane_delta:
@@ -531,7 +519,9 @@ class Tree:
         :param interval: interval to search in.
         :returns: indices of points that lie inside the interval.
 
-        >>> points = list(zip(range(-5, 6), range(10)))
+        >>> from ground.geometries import to_point_cls
+        >>> Point = to_point_cls()
+        >>> points = list(map(Point, range(-5, 6), range(10)))
         >>> tree = Tree(points)
         >>> tree.find_interval_indices(((-3, 3), (0, 1))) == []
         True
@@ -560,13 +550,16 @@ class Tree:
         :param interval: interval to search in.
         :returns: points that lie inside the interval.
 
-        >>> points = list(zip(range(-5, 6), range(10)))
+        >>> from ground.geometries import to_point_cls
+        >>> Point = to_point_cls()
+        >>> points = list(map(Point, range(-5, 6), range(10)))
         >>> tree = Tree(points)
         >>> tree.find_interval_points(((-3, 3), (0, 1))) == []
         True
-        >>> tree.find_interval_points(((-3, 3), (0, 2))) == [(-3, 2)]
+        >>> tree.find_interval_points(((-3, 3), (0, 2))) == [Point(-3, 2)]
         True
-        >>> tree.find_interval_points(((-3, 3), (0, 3))) == [(-3, 2), (-2, 3)]
+        >>> (tree.find_interval_points(((-3, 3), (0, 3)))
+        ...  == [Point(-3, 2), Point(-2, 3)])
         True
         """
         return [point for _, point in self._find_interval_items(interval)]
@@ -591,46 +584,80 @@ class Tree:
         :returns:
             indices with points in the tree that lie inside the interval.
 
-        >>> points = list(zip(range(-5, 6), range(10)))
+        >>> from ground.geometries import to_point_cls
+        >>> Point = to_point_cls()
+        >>> points = list(map(Point, range(-5, 6), range(10)))
         >>> tree = Tree(points)
         >>> tree.find_interval_items(((-3, 3), (0, 1))) == []
         True
-        >>> tree.find_interval_items(((-3, 3), (0, 2))) == [(2, (-3, 2))]
+        >>> tree.find_interval_items(((-3, 3), (0, 2))) == [(2, Point(-3, 2))]
         True
         >>> (tree.find_interval_items(((-3, 3), (0, 3)))
-        ...  == [(2, (-3, 2)), (3, (-2, 3))])
+        ...  == [(2, Point(-3, 2)), (3, Point(-2, 3))])
         True
         """
         return list(self._find_interval_items(interval))
 
-    def _find_interval_items(self, interval: Interval) -> List[Item]:
+    def _find_interval_items(self, interval: Interval) -> Iterator[Item]:
         queue = [self._root]
         push, pop = queue.append, queue.pop
         while queue:
-            node = pop()
+            node = pop()  # type: Node
             if _interval.contains_point(interval, node.point):
                 yield node.item
-            min_coordinate, max_coordinate = interval[node.axis]
-            if node.left is not NIL and min_coordinate <= node.coordinate:
+            min_coordinate, max_coordinate = interval[node.is_y_axis]
+            coordinate = node.projector(node.point)
+            if node.left is not NIL and min_coordinate <= coordinate:
                 push(node.left)
-            if node.right is not NIL and node.coordinate <= max_coordinate:
+            if node.right is not NIL and coordinate <= max_coordinate:
                 push(node.right)
 
 
 def _create_node(cls: Type[Node],
                  indices: Sequence[int],
                  points: Sequence[Point],
-                 dimension: int,
-                 axis: int) -> Union[Node, NIL]:
+                 is_y_axis: bool) -> Union[Node, NIL]:
     if not indices:
         return NIL
     indices = sorted(indices,
-                     key=lambda index: points[index][axis])
+                     key=(lambda index, projector=_PROJECTORS[is_y_axis]
+                          : projector(points[index])))
     middle_index = (len(indices) - 1) // 2
     pivot_index = indices[middle_index]
-    next_axis = (axis + 1) % dimension
-    return cls(pivot_index, points[pivot_index], axis,
-               _create_node(cls, indices[:middle_index], points, dimension,
-                            next_axis),
-               _create_node(cls, indices[middle_index + 1:], points, dimension,
-                            next_axis))
+    next_is_y_axis = not is_y_axis
+    return cls(pivot_index, points[pivot_index], is_y_axis,
+               _create_node(cls, indices[:middle_index], points,
+                            next_is_y_axis),
+               _create_node(cls, indices[middle_index + 1:], points,
+                            next_is_y_axis))
+
+
+def _to_default_node_cls() -> Type[Node]:
+    class Node:
+        __slots__ = 'index', 'point', 'is_y_axis', 'projector', 'left', 'right'
+
+        def __init__(self,
+                     index: int,
+                     point: Point,
+                     is_y_axis: bool,
+                     left: Union['Node', NIL],
+                     right: Union['Node', NIL]) -> None:
+            self.index, self.point = index, point
+            self.is_y_axis, self.projector = is_y_axis, _PROJECTORS[is_y_axis]
+            self.left, self.right = left, right
+
+        __repr__ = generate_repr(__init__)
+
+        _square_rooter = staticmethod(_to_square_rooter())
+
+        @property
+        def item(self) -> Item:
+            return self.index, self.point
+
+        def distance_to_point(self, point: Point) -> Coordinate:
+            return _points_distance(self._square_rooter, self.point, point)
+
+        def distance_to_coordinate(self, coordinate: Coordinate) -> Coordinate:
+            return abs(self.projector(self.point) - coordinate)
+
+    return Node
